<?php
// Copyright (C) 2001-2004 by Michael Earls, michael@michaelearls.com
// Copyright (C) 2005 Claus Lund, clauslund@gmail.com
// Copyright (C) 2007 Robert McLeay, robert@iseek.com.au

//------------------------------------------------------------------------
// This function returns the current microtime.
//------------------------------------------------------------------------
function get_microtime() {
	list($usec, $sec) = explode(" ", microtime());
	return ((float)$usec + (float)$sec);
}


//------------------------------------------------------------------------
// This functions verifies a username/password combination. If the
// combination exists then the function returns TRUE. If not then it
// returns FALSE.
//------------------------------------------------------------------------
function verify_login($username, $password, $link) {
	// If the username or password is blank then return FALSE.
	if(!$username || !$password) {
		return FALSE;
	}

	// Get the sha1 hash of the password and query the database.
	$pwHash = sha1($password);
	$query = "SELECT * FROM ".AUTHTABLENAME." WHERE username='".$username."' AND pwhash='".$pwHash."'";
	$result = perform_query($query, $link);

	// If the query returns one result row then return TRUE.
	if(num_rows($result) == 1) {
		return TRUE;
	}
	else {
		return FALSE;
	}
}


//------------------------------------------------------------------------
// This function verifies a username/sessionId combination. If the
// combination exists then the function returns TRUE. If not then it
// returns FALSE. If the RENEW_SESSION_ON_EACH_PAGE parameter is set then
// the functions also updates the timestamp for the session after it is
// verified.
//------------------------------------------------------------------------
function verify_session($username, $sessionId, $link) {
	// If the username or sessionId is blank then return FALSE.
	if(!$username || !$sessionId) {
		return FALSE;
	}

	// Query the database.
	$query = "SELECT * FROM ".AUTHTABLENAME." WHERE username='".$username."' 
		AND sessionid='".$sessionId."' AND exptime>now()";
	$result = perform_query($query, $link);

	// If the query returns one result row then the session is verified.
	if(num_rows($result) == 1) {
		//If RENEW_SESSION_ON_EACH_PAGE is set then update the
		// session timestamp in the database.
		if(defined('RENEW_SESSION_ON_EACH_PAGE') && RENEW_SESSION_ON_EACH_PAGE == TRUE) {
			$expTime = time()+SESSION_EXP_TIME;
			$expTimeDB = date('Y-m-d H:i:s', $expTime);
			$query = "UPDATE ".AUTHTABLENAME." SET exptime='".$expTimeDB."'
				WHERE username='".$username."'";
			perform_query($query, $link);
		}
		return TRUE;
	}
	else {
		return FALSE;
	}
}


//------------------------------------------------------------------------
// Function used to retrieve input values and if neccessary add slashes.
//------------------------------------------------------------------------
function get_input($varName) {
	$value="";
	if(isset($_COOKIE[$varName])) {
		$value = $_COOKIE[$varName];
	}
	if(isset($_GET[$varName])) {
		$value = $_GET[$varName];
	}
	if(isset($_POST[$varName])) {
		$value = $_POST[$varName];
	}

	if($value && !get_magic_quotes_gpc()) {
		if(!is_array($value)) {
			$value = addslashes($value);
		}
		else {
			foreach($value as $key => $arrValue) {
				$value[$key] = addslashes($arrValue);
			}
		}
	}

	return $value;
}


//------------------------------------------------------------------------
// Function used to validate user supplied variables.
//------------------------------------------------------------------------
function validate_input($value, $regExpName) {
	global $regExpArray;

	if(!$regExpArray[$regExpName]) {
		return FALSE;
	}

	if(is_array($value)) {
		foreach($value as $arrval) {
			if(!preg_match("$regExpArray[$regExpName]", $arrval)) {
				return FALSE;
			}
		}
		return TRUE;
	}
	elseif(preg_match("$regExpArray[$regExpName]", $value)) {
		return TRUE;
	}
	else {
		return FALSE;
	}
}


//------------------------------------------------------------------------
// This function reloads the cache with data from $table.
//------------------------------------------------------------------------
function reload_cache($table, $link) {
	$cacheHostValues = array();
	$cacheFacilityValues = array();

	// Get new cache values from $table
	$sql = "SELECT DISTINCT host FROM ".$table;
	$result = perform_query($sql, $link);
	while($row = $result->FetchRow()) {
		array_push($cacheHostValues, $row['host']);
	}

	$sql = "SELECT DISTINCT facility FROM ".$table;
	$result = perform_query($sql, $link);
	while($row = $result->FetchRow()) {
		array_push($cacheFacilityValues, $row['facility']);
	}

	// Prepare INSERT statements
	$updateTime = date("Y-m-d H:i:s");
	
	// Insert new cache values for $table
	// Not all databases support mysql-extended inserts!
	foreach($cacheHostValues as $value) {
		$insertHost = "INSERT INTO ".CACHETABLENAME." (tablename, type, value, updatetime) VALUES "
					  . "('".$table."', 'HOST', '".$value."', '".$updateTime."')";
		perform_query($insertHost, $link);
	}

	// Insert new cache values for $table
	// Not all databases support mysql-extended inserts!
	foreach($cacheFacilityValues as $value) {
		$insertFacility = "INSERT INTO ".CACHETABLENAME." (tablename, type, value, updatetime) VALUES "
						  . "('".$table."', 'FACILITY', '".$value."', '".$updateTime."')";
		perform_query($insertFacility, $link);
	}

	// Drop old cache values for $table
	$sql = "DELETE FROM ".CACHETABLENAME." WHERE tablename='".$table.
		"' AND updatetime<'".$updateTime."'";
	perform_query($sql, $link);
}


//========================================================================
// BEGIN DATABASE FUNCTIONS
//========================================================================
//------------------------------------------------------------------------
// This function connects to the database server and selects the database
// specified in the DBNAME parameter. If an error occurs then return
// FALSE.
//------------------------------------------------------------------------
function db_connect_syslog($dbUser, $dbPassword, $connType = 'P') {
	if (DBTYPE == 'postgres7' || DBTYPE == 'postgres8') {
		$server_string = DBHOST;
	}
	else {
		$server_string = DBHOST.":".DBPORT;
	}
	
	$conn = &ADONewConnection(DBTYPE);
	
	if (DBDEBUG == 'TRUE') {
		$conn->debug = true;
	}
	// Make a persistant connection if required.
	if ($connType == 'P') {
		$conn->PConnect($server_string, $dbUser, $dbPassword, DBNAME);
	} else {
		$conn->Connect($server_string, $dbUser, $dbPassword, DBNAME);
	}
	
	if($conn) {
		return $conn;
	} else {
		echo("Database connection error: " . $conn->ErrorMsg());
		return FALSE;
	}
}


//------------------------------------------------------------------------
// This functions performs the SQL query and returns a result resource. If
// an error occurs then execution is halted an the MySQL error is
// displayed.
//------------------------------------------------------------------------
function perform_query($query, $link) {
	if($link) {
		$result = &$link->Execute($query) or die('Query failed: ' . $link->ErrorMsg() .
												 'Query was: ' . $query);
	}
	else {
		die('No DB link available for query: ' . $query);
	}
	
	if (!$result) {	
		die('No DB link - result set unavailable for query: ' . $query);
	}
		
	return $result;
}


//------------------------------------------------------------------------
// This functions returns a result row as an array.
// The type can be BOTH, ASSOC or NUM.
//------------------------------------------------------------------------
function fetch_array($result, $type = 'NUM') {
	if($type == 'BOTH') {
		$returnVal = $result->GetRowAssoc();
		$result->MoveNext();
		return $returnVal;
	}
	elseif($type == 'ASSOC') {
		$returnVal = $result->GetRowAssoc();
		$result->MoveNext();
		return $returnVal;
	}
	elseif($type == 'NUM') {
		return $result->FetchRow();
	}
	else {
		die('Wrong type for fetch_array()');
	}
}


//------------------------------------------------------------------------
// This functions sets the row offset for a result resource
//------------------------------------------------------------------------
function result_seek($result, $rowNumber) {
	$result->Move($rowNumber);
}


//------------------------------------------------------------------------
// This functions returns a result row as an array
//------------------------------------------------------------------------
function num_rows($result) {
	return $result->RecordCount();
}


//------------------------------------------------------------------------
// This function checks if a particular table exists.
//------------------------------------------------------------------------
function table_exists($tableName, $link) {
	$tables = get_tables($link);
	if(array_search($tableName, $tables) !== FALSE) {
		return TRUE;
	}
	else {
		return FALSE;
	}
}


//------------------------------------------------------------------------
// This function returns an array of the names of all tables in the
// database.
//------------------------------------------------------------------------
function get_tables($link) {
	return $link->MetaTables('TABLES');
}


//------------------------------------------------------------------------
// This function returns an array with the names of tables with log data.
//------------------------------------------------------------------------
function get_logtables($link) {
	// Create an array of the column names in the default table
	$defaultFieldArray = $link->MetaColumnNames(DEFAULTLOGTABLE,$numericIndex=true);

	// Create an array with the names of all the log tables
	$logTableArray = array();
	$allTablesArray = get_tables($link);
	foreach($allTablesArray as $value) {
		$fieldArray = $link->MetaColumnNames($value,$numericIndex=true);

		// If the current array is identical to the one from the
		// DEFAULTLOGTABLE then the name is added to the result
		// array.
		$diffArray = array_diff_assoc($defaultFieldArray, $fieldArray);
		if(!$diffArray) {
			array_push($logTableArray, $value);
		}
	}
	return $logTableArray;
}
//========================================================================
// END DATABASE FUNCTIONS
//========================================================================

//========================================================================
// BEGIN ACCESS CONTROL FUNCTIONS
//========================================================================
//------------------------------------------------------------------------
// This function verifies that the user has access to a particular part
// of php-syslog-ng.
// Inputs are:
// username
// actionName
// dbLink
//
// Outputs TRUE or FALSE
//------------------------------------------------------------------------
function grant_access($userName, $actionName, $link) {
	// If ACL is not used then always return TRUE
	if(!defined('USE_ACL') || !USE_ACL || !defined('REQUIRE_AUTH') || !REQUIRE_AUTH) {
		return TRUE;
	}

	// Get user access
	$sql = "SELECT access FROM ".USER_ACCESS_TABLE." WHERE username='".$userName."' 
			AND actionname='".$actionName."'";
	$result = perform_query($sql, $link);
	$row = fetch_array($result);
	if(num_rows($result) && $row['access'] == 'TRUE') {
		return TRUE;
	}
	// Get default access
	else {
		$sql = "SELECT defaultaccess FROM ".ACTION_TABLE." WHERE actionname='".$actionName."'";
		$result = perform_query($sql, $link);
		$row = fetch_array($result);
		if($row['defaultaccess'] == 'TRUE') {
			return TRUE;
		}
		else {
			return FALSE;
		}
	}
}
//========================================================================
// END ACCESS CONTROL FUNCTIONS
//========================================================================
?>
