CREATE SEQUENCE logs_seq;

CREATE TABLE logs (
host varchar(32) default NULL,
facility varchar(10) default NULL,
priority varchar(10) default NULL,
level varchar(10) default NULL,
tag varchar(10) default NULL,
datetime timestamp default NULL,
program varchar(15) default NULL,
msg text,
seq INTEGER NOT NULL DEFAULT NEXTVAL('public.logs_seq'::text),
CONSTRAINT logs_pkey PRIMARY KEY (seq)
);

CREATE INDEX logs_host ON logs (host);
CREATE INDEX logs_program ON logs (program);
CREATE INDEX logs_datetime ON logs (datetime);
CREATE INDEX logs_priority ON logs (priority);
CREATE INDEX logs_facility ON logs (facility);

CREATE TABLE users (
username varchar(32) default NULL,
pwhash char(40) default NULL,
sessionid char(32) default NULL,
exptime timestamp default NULL,
CONSTRAINT users_pkey PRIMARY KEY (username)
);

CREATE TABLE search_cache (
tablename varchar(32) DEFAULT NULL,
type varchar(8),
value varchar(32) DEFAULT NULL,
updatetime timestamp DEFAULT NULL,
CHECK (type IN('HOST','FACILITY'))
);
CREATE INDEX search_cache_name ON search_cache (type, tablename);

CREATE TABLE user_access (
username varchar(32) DEFAULT NULL,
actionname varchar(32) DEFAULT NULL,
access varchar(5) DEFAULT 'TRUE',
CHECK (access IN('TRUE','FALSE'))
);

CREATE INDEX user_access_user_action ON user_access (username, actionname);

CREATE TABLE actions (
actionname varchar(32) NOT NULL,
actiondescr varchar(64) DEFAULT NULL,
defaultaccess varchar(5) DEFAULT 'TRUE',
CHECK (defaultaccess IN('TRUE','FALSE')),
CONSTRAINT actions_pkey PRIMARY KEY (actionname)
);

INSERT INTO actions (actionname, actiondescr, defaultaccess) VALUES ('add_user', 'Add users', 'TRUE');
INSERT INTO actions (actionname, actiondescr, defaultaccess) VALUES ('edit_user', 'Edit users (delete and change password)', 'TRUE');
INSERT INTO actions (actionname, actiondescr, defaultaccess) VALUES ('reload_cache', 'Reload search cache', 'TRUE');
INSERT INTO actions (actionname, actiondescr, defaultaccess) VALUES ('edit_acl', 'Edit access control settings', 'TRUE');

INSERT INTO users (username, pwhash) VALUES('admin', 'd033e22ae348aeb5660fc2140aec35850c4da997');

