#!/usr/bin/php
<?php
// Copyright (C) 2005 Claus Lund, clauslund@gmail.com
echo "\nStarting logrotate\n";
echo date("Y-m-d H:i:s");
$APP_ROOT = '/var/www/html';

include_once "$APP_ROOT/includes/adodb/adodb.inc.php";
include_once "$APP_ROOT/includes/common_funcs.php";
include_once "$APP_ROOT/config/config.php";

$dbLink = db_connect_syslog(DBADMIN, DBADMINPW);

// Drop temp table if it exists
// Not all db types support IF EXISTS
$tablearray = get_tables($dbLink);
$table_exists = false;
for ($count=0;$count<sizeof($tablearray);$count++) {
	if ($tablearray[$count] == "temp" . DEFAULTLOGTABLE) {
		$query = "DROP TABLE temp".DEFAULTLOGTABLE;
		perform_query($query, $dbLink);
		break;
	}
}

$today = date("Ymd");

// Create new table
$createQuery = "CREATE TABLE temp".DEFAULTLOGTABLE."(" .
				"host varchar(32) default NULL," .
				"facility varchar(10) default NULL," .
				"priority varchar(10) default NULL," .
				"level varchar(10) default NULL," .
				"tag varchar(10) default NULL," .
				"datetime timestamp default NULL," .
				"program varchar(15) default NULL," .
				"msg text," .
				"seq INTEGER NOT NULL DEFAULT NEXTVAL('public.logs_seq'::text)," .
				"CONSTRAINT logs_pkey$today PRIMARY KEY (seq)" .
				");";
perform_query($createQuery, $dbLink);

// Ok, let's add some indexes
$indexQuery = "CREATE INDEX priority_index$today ON temp".DEFAULTLOGTABLE." (priority);";
perform_query($indexQuery, $dbLink);

$indexQuery = "CREATE INDEX host_index$today ON temp".DEFAULTLOGTABLE." (host);";
perform_query($indexQuery, $dbLink);

$indexQuery = "CREATE INDEX facility_index$today ON temp".DEFAULTLOGTABLE." (facility);";
perform_query($indexQuery, $dbLink);

// Drop the merge table
if(defined('MERGELOGTABLE') && MERGELOGTABLE) {
	if (DBTYPE == 'mysql') {
		$query = "FLUSH TABLES";
		perform_query($query, $dbLink);
	}
	
	// See if the mergelog table exists
	// Not all db types support IF EXISTS
	$tablearray = get_tables($dbLink);
	$table_exists = false;
	for ($count=0;$count<sizeof($tablearray);$count++) {
		if ($tablearray[$count] == MERGELOGTABLE) {
			$query = "DROP TABLE ".MERGELOGTABLE;
			perform_query($query, $dbLink);
			break;
		}
	}
	
	
	
}

// Rename the two tables and fix up the sequences
$query = "ALTER TABLE ".DEFAULTLOGTABLE." RENAME TO ".DEFAULTLOGTABLE.$today;
perform_query($query, $dbLink);
// Remove old sequence
$query = "ALTER TABLE ".DEFAULTLOGTABLE.$today." ALTER COLUMN seq DROP DEFAULT";
perform_query($query, $dbLink);
// Rename new log table
$query = "ALTER TABLE temp".DEFAULTLOGTABLE." RENAME TO ".DEFAULTLOGTABLE;
perform_query($query, $dbLink);

// Fix permissions on new log table
$query = "GRANT INSERT ON ".DEFAULTLOGTABLE." TO syslogfeeder";
perform_query($query, $dbLink);


echo "\nLog rotate ended successfully.\n";
/*echo "Now optimizing the old logs.\n";
$query = "OPTIMIZE TABLE ".DBNAME.".".DEFAULTLOGTABLE.$today;
perform_query($query, $dbLink); */

// Re-create the merge table
if(defined('MERGELOGTABLE') || defined('LOGROTATERETENTION')) {
	echo "Getting list of log tables.\n";
	$logTableArray = get_logtables($dbLink);
}

if(defined('LOGROTATERETENTION') && LOGROTATERETENTION) {
	echo "Searching for tables to drop.\n";
	foreach($logTableArray as $value) {
		if(preg_match("([0-9]{8$row[1];}$)", $value)) {
			// determine is datestamp is old enough
			$tableDate = strrev(substr(strrev($value), 0, 8));

			$cutoffDate = date("Ymd", mktime(0, 0, 0, date("m"), date("d")-LOGROTATERETENTION, date("Y")));

			if($cutoffDate > $tableDate) {
				echo "Dropping ".$value."!\n";
				$query = "DROP TABLE ".$value;
				perform_query($query, $dbLink);
			}
		}
	}
}

//fix
if(defined('MERGELOGTABLE') && MERGELOGTABLE) {
	echo "Creating merge table.\n";

	$createQuery = "CREATE TABLE ".MERGELOGTABLE."(" .
				"host varchar(32) default NULL," .
				"facility varchar(10) default NULL," .
				"priority varchar(10) default NULL," .
				"level varchar(10) default NULL," .
				"tag varchar(10) default NULL," .
				"datetime timestamp default NULL," .
				"program varchar(15) default NULL," .
				"msg text," .
				"seq INTEGER NOT NULL," .
				"CONSTRAINT ".MERGELOGTABLE."_pkey PRIMARY KEY (seq)" .
				");";

	perform_query($createQuery, $dbLink);

	// Ok, let's add some indexes
	$indexQuery = "CREATE INDEX ".MERGELOGTABLE."_priority ON ".MERGELOGTABLE." (priority);";
	perform_query($indexQuery, $dbLink);

	$indexQuery = "CREATE INDEX ".MERGELOGTABLE."_host ON ".MERGELOGTABLE." (host);";
	perform_query($indexQuery, $dbLink);

	$indexQuery = "CREATE INDEX ".MERGELOGTABLE."_facility ON ".MERGELOGTABLE." (facility);";
	perform_query($indexQuery, $dbLink);


	foreach($logTableArray as $value) {
		$query = "INSERT INTO ".MERGELOGTABLE." SELECT * FROM $value";
		perform_query($query, $dbLink);
	}


}
echo "\n".date("Y-m-d H:i:s")."\n";
echo "All done!\n";
?>
